﻿Imports System.IO
Imports System.Text

Public Class Response
    Public Property StatusCode() As Integer
        Get
            Return m_StatusCode
        End Get
        Private Set
            m_StatusCode = Value
        End Set
    End Property
    Private m_StatusCode As Integer

    Public ReadOnly Property Reason() As String
        Get
            Return If(StatusCode > 0, statusCodes(StatusCode), Nothing)
        End Get
    End Property

    Public Property Content() As String
        Get
            Return m_Content
        End Get
        Private Set
            m_Content = Value
        End Set
    End Property
    Private m_Content As String

    Private ReadOnly statusCodes As New Dictionary(Of Integer, String)() From {
            {200, "OK"},
            {401, "Unauthorized"},
            {404, "Not found"},
            {501, "Not implemented"}
        }

    Private request As Request

    Public Shared Function Create(request As Request) As Response
        Dim response = New Response()
        response.request = request

        If request.HttpMethod.Equals("GET") Then
            response.[GET](request)
        ElseIf request.HttpMethod.Equals("POST") Then
            response.POST(request)
        Else
            response.notImplemented()
        End If

        Return response
    End Function

    Private Sub [GET](request As Request)
        Dim filename = request.LocalUrl.Trim("/"c)

        StatusCode = If(File.Exists(filename), 200, 404)
        Content = If(File.Exists(filename), File.ReadAllText(filename), "")
    End Sub

    Private Sub POST(request As Request)
        [GET](request)
    End Sub

    Private Sub notImplemented()
        StatusCode = 501
    End Sub

    Public Overrides Function ToString() As String
        If Not authenticated() Then
            StatusCode = 401
        End If

        Dim response = String.Format("HTTP/1.1 {0} {1}" & vbCr & vbLf, StatusCode, Reason)

        response += "Connection: close" & vbCr & vbLf

        If Not authenticated() Then
            response += "WWW-Authenticate: Basic realm=SecArea" & vbCr & vbLf & vbCr & vbLf
        Else
            Dim filetype = Path.GetExtension(request.LocalUrl).TrimStart("."c)
            If filetype = "php" Then
                filetype = "html"
            End If
            response += "Content-Type: text/" + filetype + ";charset=utf-8" & vbCr & vbLf
            response += "Set-Cookie: Computer=" + request.IPEndPoint.Address.ToString() + vbCr & vbLf
            response += vbCr & vbLf

            response += New Script(Content, request).ToString()
        End If

        Return response
    End Function

    Private Function authenticated() As Boolean
        If Not request.LocalUrl.Contains("secure") Then
            Return True
        End If

        Dim authorization = If(request.Header.ContainsKey("Authorization"), request.Header("Authorization"), Nothing)

        If authorization Is Nothing Then
            Return False
        End If

        Dim bytes = Convert.FromBase64String(authorization.Replace("Basic ", ""))
        Dim password = Encoding.UTF8.GetString(bytes)

        Return password = "user:secure"
    End Function

End Class
