﻿Imports System.Data.Entity
Imports System.Net
Imports FlightSearch.Models

Namespace Controllers
    Public Class FlightController
        Inherits System.Web.Mvc.Controller

        Private db As New FlightSearchContext

        ' GET: Flight
        Public Function Index(query As String) As ActionResult
            Dim flights = Queryable.AsQueryable(db.FlightRoutes)

            If Not String.IsNullOrEmpty(query) Then
                flights = flights _
                    .Where(Function(f) f.Date.ToString().Contains(query) _
                            OrElse f.FlightNo.Contains(query) _
                            OrElse f.FromAirport.Contains(query) _
                            OrElse f.ToAirport.Contains(query))
            End If
            Return View(flights.ToList())
        End Function


        ' GET: Flight/Details/5
        Function Details(ByVal id As Integer?) As ActionResult
            If IsNothing(id) Then
                Return New HttpStatusCodeResult(HttpStatusCode.BadRequest)
            End If
            Dim flightRoute As FlightRoute = db.FlightRoutes.Find(id)
            If IsNothing(flightRoute) Then
                Return HttpNotFound()
            End If
            Return View(flightRoute)
        End Function

        ' GET: Flight/Create
        <Authorize(Users:="muster@beispiel.de, adm@beispiel.de")>
        Function Create() As ActionResult
            Return View()
        End Function

        ' POST: Flight/Create
        'Aktivieren Sie zum Schutz vor übermäßigem Senden von Angriffen die spezifischen Eigenschaften, mit denen eine Bindung erfolgen soll. Weitere Informationen 
        'finden Sie unter http://go.microsoft.com/fwlink/?LinkId=317598.
        <HttpPost()>
        <ValidateAntiForgeryToken()>
        Public Function Create(flightRoute As FlightRoute) As ActionResult
            If ModelState.IsValid Then
                db.FlightRoutes.Add(flightRoute)
                db.SaveChanges()
                Return RedirectToAction("Index")
            End If
            Return View(flightRoute)
        End Function

        ' GET: Flight/Edit/5
        Function Edit(ByVal id As Integer?) As ActionResult
            If IsNothing(id) Then
                Return New HttpStatusCodeResult(HttpStatusCode.BadRequest)
            End If
            Dim flightRoute As FlightRoute = db.FlightRoutes.Find(id)
            If IsNothing(flightRoute) Then
                Return HttpNotFound()
            End If
            Return View(flightRoute)
        End Function

        ' POST: Flight/Edit/5
        'Aktivieren Sie zum Schutz vor übermäßigem Senden von Angriffen die spezifischen Eigenschaften, mit denen eine Bindung erfolgen soll. Weitere Informationen 
        'finden Sie unter http://go.microsoft.com/fwlink/?LinkId=317598.
        <HttpPost()>
        <ValidateAntiForgeryToken()>
        Function Edit(<Bind(Include:="Id,FlightNo,Date,From,To,FromAirport,ToAirport,Hints")> ByVal flightRoute As FlightRoute) As ActionResult
            If ModelState.IsValid Then
                db.Entry(flightRoute).State = EntityState.Modified
                db.SaveChanges()
                Return RedirectToAction("Index")
            End If
            Return View(flightRoute)
        End Function

        ' GET: Flight/Delete/5
        Function Delete(ByVal id As Integer?) As ActionResult
            If IsNothing(id) Then
                Return New HttpStatusCodeResult(HttpStatusCode.BadRequest)
            End If
            Dim flightRoute As FlightRoute = db.FlightRoutes.Find(id)
            If IsNothing(flightRoute) Then
                Return HttpNotFound()
            End If
            Return View(flightRoute)
        End Function

        ' POST: Flight/Delete/5
        <HttpPost()>
        <ActionName("Delete")>
        <ValidateAntiForgeryToken()>
        Function DeleteConfirmed(ByVal id As Integer) As ActionResult
            Dim flightRoute As FlightRoute = db.FlightRoutes.Find(id)
            db.FlightRoutes.Remove(flightRoute)
            db.SaveChanges()
            Return RedirectToAction("Index")
        End Function

        Protected Overrides Sub Dispose(ByVal disposing As Boolean)
            If (disposing) Then
                db.Dispose()
            End If
            MyBase.Dispose(disposing)
        End Sub
    End Class
End Namespace
